#
# This file is a part of the RPA program (Robust Probabilistic
# Averaging), see http://www.cis.hut.fi/projects/mi/software/RPA/
#
# Copyright (C) 2008-2010 Leo Lahti (leo.lahti@iki.fi)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License 2 for more details.
# 

RPA.pointestimate <- function (abatch,
                                 sets = NULL,
                               myseed = 101, 
                               priors = NULL,
                              epsilon = 1e-2, 
                                 cind = 1,
                        sigma2.method = "robust",
                             d.method = "fast",
                              verbose = TRUE,
                            bg.method = "rma",
                 normalization.method = "quantiles.robust",
                                  cdf = NULL,
                                alpha = NULL,
                                 beta = NULL)                                      
{

#
# Find posterior mode for RPA model parameters d (mean) and sigma2 (variances)
#
# rpa.results <- RPA.pointestimate(abatch)
#

  #################################################################

  # PREPROCESSING

  #################################################################

  #Set random seed
  set.seed( myseed )

  # Set alternative CDF environment if given
  if (!is.null(cdf)) { abatch@cdfName <- cdf }
      
  # Preprocessing
  preproc <- RPA.preprocess(abatch, cind, bg.method, normalization.method, cdf)			   
  fcmat <- preproc$fcmat  # probe-level differential expressions
   cind <- preproc$cind   # index of the control array
  set.inds <- preproc$set.inds   # index of the control array

  #################################################################

  # ESTIMATE PROBE RELIABILITY AND DIFFERENTIAL GENE EXPRESSION

  #################################################################

  # Number of arrays except control
  T <- ncol(fcmat)

  # Check names and number for the investigated probesets
  # if my.sets not specified, take all sets in abatch
  if (is.null(sets)) { sets <- geneNames(abatch) } 

  if (!all(sets %in% probeNames(abatch))) {
    warning("Warning: not all probesets are included in the abatch object. Running the test only for included probesets!")
    sets <- sets[sets %in% probeNames(abatch)]
  } else {}
  
  Nsets <- length(sets) 

  ## Matrices for storing the results
  d.results <- array(NA, dim = c(Nsets, T))
  rownames(d.results) <- sets
  colnames(d.results) <- colnames(fcmat)

  sigma2.results <- vector(length = Nsets, mode = "list")
  names(sigma2.results) <- sets

  if (!is.null(priors) && (!is.null(alpha) || !is.null(beta))) {
    stop("Specify either priors OR alpha, beta- both cannot be specified at the same time!")
  }
  
  for (i in 1:Nsets) {

    set <- sets[[i]]
  
    if (verbose) {message(paste("Computing probeset", set, ":", i, "/", Nsets, "...\n"))}

    # Find probe (pm) indices for this set
    pmindices <- set.inds[[set]]
    
    # Number of probes for this probeset
    P <- length(pmindices)
    
    # Get chips x probes matrix of probe-wise fold-changes
    S <- t(fcmat[pmindices, ])
    
    # Pick the priors for this set (gives NULL if no prior has been defined)
    if (!is.null(priors)) {
      alpha <- priors[[set]]$alpha 
      beta  <- priors[[set]]$beta
    }
    
    res <- RPA.iteration(S, epsilon, alpha, beta, sigma2.method, d.method)
                         
    #Store results
    d.results[i, ] <- res$d
    sigma2.results[[i]] <- res$sigma2
  
  }

  # Create new class instance
  rpa.res <- new("rpa", list(d = d.results, sigma2 = sigma2.results, cind = cind, sets = sets, data = fcmat, cdf = cdf, abatch = abatch))

  # return result class
  rpa.res
}

